<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform" version="1.0">
  <xsl:param name="cg_name" select="'busy_range_cg'" />
  <xsl:param name="cp_name" select="''" />
  <xsl:param name="target_name" select="''" />
  <xsl:param name="instance_name" select="''" />
  
  <xsl:template match="/">
    <xsl:apply-templates select="testbench"/>
  </xsl:template>
  
  <xsl:template match="testbench">
    <div>
      <xsl:apply-templates select="cg[@name=$cg_name]" />
    </div>
  </xsl:template>
  
  <xsl:template match="cg">
    <xsl:choose>
      <xsl:when test="$instance_name!=''"><xsl:apply-templates select="inst[@instname=$instance_name]" /></xsl:when>
      <xsl:otherwise><xsl:apply-templates select="cumulative" /></xsl:otherwise>
    </xsl:choose>
  </xsl:template>

  <xsl:template name="ranking">
    <xsl:param name="score" />
    <xsl:choose>
      <xsl:when test="$score=100"><xsl:attribute name="class">Lv5</xsl:attribute></xsl:when>
      <xsl:when test="$score&gt;=80"><xsl:attribute name="class">Lv4</xsl:attribute></xsl:when>
      <xsl:when test="$score&gt;=60"><xsl:attribute name="class">Lv3</xsl:attribute></xsl:when>
      <xsl:when test="$score&gt;=30"><xsl:attribute name="class">Lv2</xsl:attribute></xsl:when>
      <xsl:when test="$score&gt; 0"><xsl:attribute name="class">Lv1</xsl:attribute></xsl:when>
      <xsl:when test="$score=0"><xsl:attribute name="class">Lv0</xsl:attribute></xsl:when>
      <xsl:otherwise></xsl:otherwise>
    </xsl:choose>    
  </xsl:template>

  
  <xsl:template match="cumulative|inst">
    <table summary="cg_table" class="infotable">
      <caption>current coverage group	</caption>
      <thead>
	<tr>
	  <th scope="col" class="namecol">name</th>
	  <th scope="col">class</th>
	  <xsl:if test="$instance_name!=''">
	    <th scope="col">instance</th>
	  </xsl:if>
	  <th scope="col">description</th>
	  <th scope="col" class="countcol">weight</th>
	  <th scope="col" class="scorecol">score</th>
	</tr>
      </thead>
      <tbody>
	<tr>
	  <th>
	    <xsl:choose>
	      <xsl:when test="$cp_name">
		<a>
		  <xsl:attribute name="onclick">showCoverageGroup('<xsl:value-of select="$cg_name"/>')</xsl:attribute>
		  <xsl:value-of select="@name" />
		</a>
	      </xsl:when>
	      <xsl:otherwise>
		<xsl:value-of select="@name" />
	      </xsl:otherwise>
	    </xsl:choose>
	  </th>
	  <td>
	    <xsl:value-of select="../cumulative/@class" />
	  </td>
	  <xsl:if test="$instance_name!=''">
	    <td><xsl:value-of select="$instance_name"/></td>
	  </xsl:if>
	  <td><xsl:value-of select="@description" /></td>
	  <td><xsl:value-of select="@weight" /></td>
	  <td>
	    <xsl:call-template name="ranking"><xsl:with-param name="score" select="@score"/></xsl:call-template>
	    <xsl:value-of select="@score" />
	  </td>
	</tr>
      </tbody>
    </table>
    <xsl:apply-templates select="cp[@name=$cp_name]|cc[@name=$cp_name]"/>
  </xsl:template>

  <xsl:template match="cp">
    <xsl:call-template name="create_cc_cp_table" />
  </xsl:template>

  <xsl:template match="cc">
    <xsl:call-template name="create_cc_cp_table" />
    <xsl:call-template name="create_cc_detail" />
  </xsl:template>

  <xsl:template name="create_cc_cp_table">
    <table summary="cp_table" class="infotable">
      <caption>current coverage point</caption>
      <thead>
	<tr>
	  <th scope="col" class="namecol">name</th>
	  <th scope="col">description</th>
	  <th scope="col" class="countcol">#expected</th>
	  <th scope="col" class="countcol">#OKs</th>
	  <th scope="col" class="countcol">weight</th>
	  <th scope="col" class="scorecol">score</th>
	</tr>
      </thead>
      <tbody>
	<tr>
	  <th><xsl:value-of select="@name" /></th>
	  <td><xsl:value-of select="@description" /></td>
	  <td><xsl:value-of select="@weight" /></td>
	  <td><xsl:value-of select="@expected" /></td>
	  <td><xsl:value-of select="@ok" /></td>
	  <td>
	    <xsl:call-template name="ranking"><xsl:with-param name="score" select="@score"/></xsl:call-template>
	    <xsl:value-of select="@score" />
	  </td>
	</tr>
      </tbody>
    </table>
  </xsl:template>

  <xsl:template name="create_cc_detail">
    <table summary="cc_detail" class="infotable" style="width:60%; margin:auto 0% auto auto">
      <caption>in detail</caption>
      <thead>
	<tr>
	  <th scope="col">type</th>
	  <th scope="col" class="countcol">#expected</th>
	  <th scope="col" class="countcol">#OKs</th>
	  <th scope="col" class="scorecol">score</th>
	</tr>
      </thead>
      <tbody>
	<xsl:apply-templates select="*"/>
      </tbody>
    </table>
  </xsl:template>

  <xsl:template match="data|namedbins">
    <tr>
      <xsl:choose>
	<xsl:when test="name(.)='namedbins'">
	  <th><xsl:value-of select="'Named Bins'"/></th>
	</xsl:when>
	<xsl:when test="name(.)='data'">
	  <th>
	    <a>
	      <xsl:attribute name="href">
		VerifiUI_AutoXcView.html?target=<xsl:value-of select="$target_name"/>&amp;cg=<xsl:value-of select="$cg_name"/>&amp;cp=<xsl:value-of select="$cp_name"/>&amp;instance=<xsl:value-of select="$instance_name"/>
	      </xsl:attribute>
	      <xsl:value-of select="'Auto Generated Bins'"/>
	    </a>
	  </th>
	</xsl:when>
      </xsl:choose>
      <td><xsl:value-of select="@expected"/></td>
      <td><xsl:value-of select="@ok"/></td>
      <td>
	<xsl:call-template name="ranking"><xsl:with-param name="score" select="@score"/></xsl:call-template>
	<xsl:value-of select="@score"/>
      </td>
    </tr>
  </xsl:template>
</xsl:stylesheet>
