<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform" version="1.0">
  <xsl:param name="cg_name" select="''" />
  <xsl:param name="show_weight_zero" select="'no'" />
  <xsl:param name="sortby" select="'@name'" />
  <xsl:template match="/">
    <div>
      <xsl:apply-templates select="testbench"/>
    </div>
  </xsl:template>
  
  <xsl:template match="testbench">
    <table summary="menu_table" class="menu_table" style="margin-bottom:10pt">
      <caption>navigation</caption>
      <tbody>
	<tr style="font-style:bold">
	  <th scope="col">
	    <a>
	      <xsl:attribute name="onclick">showRoot()</xsl:attribute>
	      Total
	    </a>
	  </th>
	  <th scope="col">
	    <xsl:call-template name="ranking"><xsl:with-param name="score" select="@score_cumulative"/></xsl:call-template>
	    <xsl:value-of select="@score_cumulative" />
	  </th>
	</tr>
      </tbody>
    </table>
    <table summary="menu_table" class="menu_table">
      <tbody>
	<tr>
	  <th scope="col">
	    name
	    <a>
	      <xsl:attribute name="onclick">menuSortBy('@name')</xsl:attribute>
	      v
	    </a>
	  </th>
	  <th scope="col">
	    score
	    <a>
	      <xsl:attribute name="onclick">menuSortBy('@score')</xsl:attribute>
	      v
	    </a>
	  </th>
	</tr>
	
	<xsl:choose>
	  <xsl:when test="$sortby='@name'">
	    <xsl:for-each select="cg/cumulative">
	      <xsl:sort select="@name" data-type="text" order="ascending"/>
	      <xsl:apply-templates select="." />
	    </xsl:for-each>
	  </xsl:when>
	  <xsl:when test="$sortby='@score'">
	    <xsl:for-each select="cg/cumulative">
	      <xsl:sort select="@score" data-type="number" order="ascending"/>
	      <xsl:apply-templates select="." />
	    </xsl:for-each>
	  </xsl:when>
	  <xsl:otherwise>
	    <xsl:for-each select="cg/cumulative">
	      <xsl:apply-templates select="." />
	    </xsl:for-each>
	  </xsl:otherwise>
	</xsl:choose>
      </tbody>
    </table>
  </xsl:template>
  
  
  <xsl:template name="ranking">
    <xsl:param name="score" />
    <xsl:choose>
      <xsl:when test="$score=100"><xsl:attribute name="class">Lv5</xsl:attribute></xsl:when>
      <xsl:when test="$score&gt;=80"><xsl:attribute name="class">Lv4</xsl:attribute></xsl:when>
      <xsl:when test="$score&gt;=60"><xsl:attribute name="class">Lv3</xsl:attribute></xsl:when>
      <xsl:when test="$score&gt;=30"><xsl:attribute name="class">Lv2</xsl:attribute></xsl:when>
      <xsl:when test="$score&gt; 0"><xsl:attribute name="class">Lv1</xsl:attribute></xsl:when>
      <xsl:when test="$score=0"><xsl:attribute name="class">Lv0</xsl:attribute></xsl:when>
      <xsl:otherwise></xsl:otherwise>
    </xsl:choose>    
  </xsl:template>

  <xsl:template match="cumulative">
    <xsl:if test="@weight&gt;0 or $show_weight_zero='yes'">
      <tr>
	<th scope="row">
	  <a>
	    <xsl:attribute name="onclick">showCoverageGroup('<xsl:value-of select="@name"/>')</xsl:attribute>
	    <xsl:value-of select="@name" />
	  </a>

	  <xsl:choose>
	    <xsl:when test="@name=$cg_name">
	      <a>
		<xsl:attribute name="onclick">changeMenuToRoot()</xsl:attribute>
		&lt;&lt;
	      </a>
	    </xsl:when>
	    <xsl:otherwise>
	      <a>
		<xsl:attribute name="onclick">changeMenuToCps('<xsl:value-of select="@name"/>')</xsl:attribute>
		&gt;&gt;
	      </a>
	    </xsl:otherwise>
	  </xsl:choose>
	</th>
	<td>
	  <xsl:call-template name="ranking"><xsl:with-param name="score" select="@score"/></xsl:call-template>
	  <xsl:value-of select="@score" />
	</td>
      </tr>
      <xsl:if test="@name=$cg_name">
	<xsl:apply-templates />
      </xsl:if>
    </xsl:if>
  </xsl:template>
  
  <xsl:template match="cp|cc">
    <xsl:if test="@weight&gt;0 or $show_weight_zero='yes'">
      <tr>
	<th scope="row" style="text-align:right;">
	  <a style="font-size:80%">
	    <xsl:attribute name="onclick">showCoveragePoint('<xsl:value-of select="$cg_name" />','<xsl:value-of select="@name"/>')</xsl:attribute>
	    <xsl:value-of select="@name" />
	  </a>
	</th>
	<td>
	  <xsl:call-template name="ranking"><xsl:with-param name="score" select="@score"/></xsl:call-template>
	  <xsl:value-of select="@score" />
	</td>
      </tr>
    </xsl:if>
  </xsl:template>
</xsl:stylesheet>