#-----------------------------------------------------------------------
# 
# @file vericlear.py
# @brief vericlear processor engine
# 
# 
# Copyright (C) 2007-2012 Verifore, Inc.
# All rights reserved. Property of Verifore, Inc.
# Restricted rights to use, duplicate or disclose
# this code are granted through contract.
# 
#-----------------------------------------------------------------------
from lxml import etree
import re
from sys import argv,stdout,stdin, stderr
import itertools

pathToNode = {}
maintree = None
structure_ready = False
cross_coverage_frames = {}
cross_coverage_frame_indice = {}

#vcs_mode = False
output = None
verbose = False

def errprint(*args):
  stderr.write(" ".join(map(str,args)) + "\n")
    

class CrossCoverageFrame(object):
  def __init__(self, elem):
    assert(elem.tag == 'cc')
    self.bnList = []
    p = elem.getparent()
    ccname = elem.attrib["name"]
    cgname = p.attrib.get("name") or p.attrib["instname"]
    cross_coverage_frame_indice[(cgname, ccname)] = self
    
    cp_names = [cp.attrib["name"] for cp in elem.xpath("../cp")]

    crosspoints = elem.find("crosspoints")
    self.axis_names = [axis.attrib["name"] for axis in crosspoints.xpath("axis")]
    axis_ids = [cp_names.index(axis_name) for axis_name in self.axis_names]
    crosspoints.attrib["covrefdefs"] = " ".join([str(id) for id in axis_ids])

    for axis_name in self.axis_names:
      bn_names = [bn.attrib["name"] for bn in elem.xpath("../cp[@name='%s']/bn"%axis_name)]
      self.bnList.append(bn_names)

    datlen = reduce(lambda x,y: x*len(y), self.bnList, 1)

    if elem.find('data')!=None and elem.find('data').text !="":
      self.dat = map(lambda d:int(d), elem.find('data').text.strip().split(' '))
      assert(len(self.dat) == datlen)
    else:
      self.dat = [-3] * datlen

      valid_bin_list = []
      non_illegal_bin_list = []
      for axis_name in self.axis_names:
        valid_bn_flags = []
        non_illegal_bn_flags = []
        for i, bn in enumerate(elem.xpath("../cp[@name='%s']/bn"%axis_name)):
          b_illegal = bn.attrib.get('illegal') == 'true'
          b_valid = (not b_illegal) and bn.attrib.get('ignored') != 'true'
          if not b_illegal: non_illegal_bn_flags.append(i)
          if b_valid:     valid_bn_flags.append(i)
        valid_bin_list.append(valid_bn_flags)
        non_illegal_bin_list.append(non_illegal_bn_flags)


      def recursive_set_valid_bin(i, idx, list, val):
        idx *= len(self.bnList[i])
        for j in list[i]:
          next = idx + j
          if i == len(list)-1: self.dat[next] = val
          else: recursive_set_valid_bin(i+1, next, list, val)

      recursive_set_valid_bin(0, 0, non_illegal_bin_list, -2)
      recursive_set_valid_bin(0, 0, valid_bin_list, 0)
      elem.append(maintree.makeelement('data'))


  def getDataString(self): return " ".join([str(d) for d in self.dat])

  def getId(self, bn_names):
    state = []
    for i,bn_name in enumerate(bn_names):
      assert bn_name in self.bnList[i], "'%s' not in (%s)"%(bn_name, ",".join(self.bnList[i]))
      state.append(self.bnList[i].index(bn_name))
    return self.coordToId(state)

  def coordToId(self, l):
    ret = l[0]
    for i in xrange(1, len(self.bnList)):
      ret = ret * len(self.bnList[i]) + l[i]
    assert ret<len(self.dat), "Return Id out of range. %s->%d"%(str(l), ret)
    return ret
  
  def addVal(self, val, bn_names):
    id = self.getId(bn_names)
    if self.dat[id] < 0:
      errprint(bn_names, map(lambda x,y: x.index(y), self.bnList, bn_names))
      raise Exception("Adding count to invalid bin.%d"%self.dat[id])
    self.dat[id] += val
  
  def exclude(self, bn_names):
    id = self.getId(bn_names)
    if self.dat[id] not in (-2,-3):
      raise Exception("Excluding Valid Bin.")
    self.dat[id] = -2
  
  def setNamedBin(self, bn_names_ary):
    for bn_names in bn_names_ary:
      id = self.getId(bn_names)
      #if self.dat[id] not in (0,-1, -2, -3):
        #raise Exception("Naming Valid Bin. %d,%s"%(id, str(bn_names)))
      self.dat[id] = -1

  def modifyAndSumBinDatas(self, axes, combinations, new_value):
    def pos(cp_id, bn_name):
      return self.bnList[cp_id].index(bn_name)

    def getIdCombinations(combinations):
      def getSingleIdCombination(combination):
        id_combination = []
        for i,axis in enumerate(self.axis_names):
          if axis not in axes:
            id_combination.append(range(0, len(self.bnList[i])))
          else:
            span = combination[axes.index(axis)]
            if span == "*":
              id_combination.append(range(0, len(self.bnList[i])))
            elif isinstance(span, str) or isinstance(span, unicode):
              id_combination.append((pos(i, span), ))
            elif isinstance(span, list):
              id_combination.append(tuple(map(lambda bn: pos(i,bn), span)))
            elif isinstance(span, dict) and "from" in span and "to" in span:
              id_span = (pos(i, span["from"]), pos(i, span["to"]))
              id_combination.append(
                tuple(range(min(id_span), max(id_span)+1))
              )
            else:
              raise ValueError("cannot understand : " + str(span))
        return id_combination

      return [getSingleIdCombination(comb) for comb in combinations]

    sum = [0]
    def recursiveModifyBin(id_combination, current_list, i):
      if i == len(id_combination):
        id = self.coordToId(current_list)
        if(self.dat[id]>0): sum[0] += self.dat[id]
        self.dat[id] = new_value
        return
      for id in id_combination[i]:
        recursiveModifyBin(id_combination, current_list + [id], i+1)
      
    try:
      for id_combination in getIdCombinations(combinations):
        recursiveModifyBin(id_combination, [], 0)
      return sum[0]
        
    except ValueError as e:
      errprint("No such combinations and/or axes")
      errprint("axes = %s"%str(axes))
      errprint("combinations = %s"%str(combinations))
      errprint("====\n"+e.message+"\n====")
      raise Exception("Modification Error")
    finally:
      pass
        
      

def scanStream(stream):
  global pathToNode, maintree, linesForLater
  assert(maintree != None)
  typedic =  {'COVERGROUP':'cg', 'COVERINSTANCE':'inst', 'COVERPOINT':'cp', 'CROSS':'cc'}

  def getParentPath(path):
    """Get path string of parent."""
    path = "/".join(path.split("/")[:-1])
    while path.endswith("\\"): ## avoid escaped /
      path = getParentPath(path)
    return path

  def analyzeDataLine(line):
    """Chop datas into dictionary."""
    l = []
    split = reversed(re.findall("[^,]+",line[3:]))
    word = ""
    for s in reversed(line[3:].split(",")):
      word = s + "," + word if word != "" else s
      m = re.match("^#?(\w+)#?=(.*)$", word)
      if m:
        l.append((m.group(1), m.group(2)))
        word = ""
    ret = dict(l)

    return ret

  def scanCrossCov(args, elem):
    """Analyze Cross Coverage Scope and create corresponding XML nodes."""
    assert(elem.tag == 'cc')
    crosses = re.findall("(\w+)\W*", args["CROSSES"])
    crosspoints = maintree.makeelement('crosspoints')

    for cross_axis in crosses:
      axis = maintree.makeelement('axis')
      axis.attrib["name"] = cross_axis
      axis.attrib["n_bins"] = str(len(elem.xpath("../cp[@name='%s']/bn"%cross_axis)))
      crosspoints.append(axis)
    elem.append(crosspoints)


  def scanScope(line):
    """Analyze Scope and create XML nodes."""
    args = analyzeDataLine(line)
    scopetype = args.get("scopetype")

    path = args["path"]
    if path in pathToNode: return ##Node already exists

    parent_path  = getParentPath(path)
    parent_node = pathToNode.get(parent_path)
    name = path.split("/")[-1]

    if scopetype == 'COVERGROUP':
      cg_elem  = maintree.makeelement('cg')
      cum_elem = maintree.makeelement('cumulative')
      cg_elem.append(cum_elem)

      cg_elem.attrib["name"] = name
      cum_elem.attrib["name"] = name
      cum_elem.attrib["class"] = parent_path.split("/")[-1]
      if args.get("COMMENT") != None:
        cum_elem.attrib["description"] = args["COMMENT"].strip('"')
      if args.get("weight") != None:
        cum_elem.attrib["weight"] = args["weight"]
      pathToNode[path] = cg_elem
      maintree.append(cg_elem)

    elif scopetype in ('COVERPOINT', 'CROSS'):
      assert parent_node != None , "CANNOT FIND PARENT NODE %s,%s"%(path,parent_path)
      assert parent_node.tag in ('cg', 'inst'), "parent_node.tag is %s(%s)."%(parent_node.tag, parent_node.attrib["name"])
      if parent_node.tag == 'cg':
        parent_node = parent_node.find('cumulative')
      
      elem = maintree.makeelement({"COVERPOINT":'cp', 'CROSS':'cc'}[scopetype])
      
      elem.attrib["name"] = name
      if args.get("COMMENT") != None:
        elem.attrib["description"] = args["COMMENT"]
      if args.get("weight") != None:
        elem.attrib["weight"] = args["weight"]
      if args.get("ATLEAST") != None:
        elem.attrib["at_least"] = args["ATLEAST"]
      pathToNode[path] = elem
      parent_node.append(elem)

      if scopetype == 'CROSS':
        scanCrossCov(args, elem)
      else: ##scopetype=='COVERPOINT'
        assert len(elem.xpath("../cc/data"))==0, "Modified covergroup after cross bn counted."

    elif scopetype == 'COVERINSTANCE':
      assert parent_node != None, "parent_node(%s) not found. curret=%s"%(parent_path, path)
      assert parent_node.tag == 'cg', "parent_node.tag is %s"%parent_node.tag
      elem = maintree.makeelement('inst')
      elem.attrib["instname"] = name.strip()
      if args.get("COMMENT") != None:
        elem.attrib["description"] = args["COMMENT"]
      if args.get("weight") != None:
        elem.attrib["weight"] = args["weight"]
      pathToNode[path] = elem
      parent_node.append(elem)

    else:
      pass

  def scanBin(line):
    args = analyzeDataLine(line)
    path = args["path"]
    parent_node = pathToNode.get(path)
    assert args.get("covname") != None
    def createNewBin(parent_node, bn_name, count_str, b_ignored, b_illegal):
      elem = maintree.makeelement("bn")
      elem.attrib["name"] = bn_name
      elem.attrib["count"] = count_str
      if b_ignored: elem.attrib["ignored"] = "true"
      if b_illegal: elem.attrib["illegal"] = "true"
      parent_node.append(elem)

    if parent_node == None:
      return False
    elif parent_node.tag == "cp":
      elem = parent_node.find("bn[@name='%s']"%args["covname"])
      if elem != None:
        val = int(elem.attrib["count"]) + int(args.get("val"))
        elem.attrib["count"] = str(val)
      else:
        b_ignored = "ignored" in args and args["ignored"] != "0"
        b_illegal = "illegal" in args and args["illegal"] != "0"
        createNewBin(parent_node, args["covname"], args.get("val"), b_ignored, b_illegal)

    elif parent_node.tag == "cc":
      frame = cross_coverage_frames.get(path)
      if frame == None:
        frame = CrossCoverageFrame(parent_node)
        cross_coverage_frames[path] = frame

      if "BINRHS" not in args or args["BINRHS"] == '': ## auto cross bin
        covname = args["covname"]
        if re.match("<.*?>", covname):
          states = [covname.strip("<>").split(',')]
        elif re.match("^\[(:?\[[^\]]+\],?)+\]$", covname):
          combinations = [re.findall("[^\[, ]+",s) for s in re.findall("\[([^\]]+)\]", covname)]
          states = itertools.product(*combinations)
        elif re.match("^\[.+,\]$", covname):
          states = [covname[1:-1].strip(",").split(",")]
        else:
          assert False, "wrong covname format:" + covname
        for state in states:
          if args.get("excluded"): frame.exclude(state)
          else: frame.addVal(int(args["val"]), state)

        excluded = args.get("excluded")
        v = int(args["val"])
        for state in states:
          if excluded: frame.exclude(state)
          frame.addVal(v, state)

      else: ## named cross bin
        elem = parent_node.find("bn[@name='%s']"%args["covname"])
        if elem != None:
          val = int(elem.attrib["count"]) + int(args.get("val"))
          elem.attrib["count"] = str(val)
        else:
          createNewBin(parent_node, args["covname"], args.get("val"), args.has_key("ignored"), args.has_key("illegal"))
          states = re.findall("<([^>]+)>;?", args["BINRHS"])
          frame.setNamedBin([re.findall("(\w[^,]*),?", state) for state in states])

    return True
    

  pathToNode = {}
  binlines = []

  ## SCAN SC
  ErrorLines = []
  first_line = stream.readline()
  if not first_line.startswith("#VERICLEAR"):
    raise Exception("Wrong File Type: Input file not created by vericlear tools")
  for line in stream:
    line = line.strip()
    node_type = line[:2]
    ret = True
    try:
      if   node_type == "SC": scanScope(line)
      elif node_type == "BN": ret = scanBin(line)
      else: ret = False
    #except Exception as e:
      #errprint(line)
    finally:
      pass
    if ret == False:
      if verbose: errprint("NOT PROCESSED: " + line)
      ErrorLines.append(line)
  if(len(ErrorLines)>0):
    ##for k in pathToNode: print(k)
    raise Exception("%d lines not properly processed"%len(ErrorLines))
    

def doScoring():
  """write scoring datas"""
  def scoring_bns(elem, at_least):
    excluded = ok = expected = 0
    for bn in elem.xpath("bn"):
      if "ignored" in bn.attrib or "illegal" in bn.attrib or bn.attrib.get("excl") == "true":
        excluded += 1
      else:
        expected += 1
        if bn.attrib.has_key("count") and int(bn.attrib.get("count"))>=at_least:
          ok += 1
    return expected, excluded, ok
  
  def scoring_auto_crosses(elem, at_least):
    excluded = ok = expected = named = illegal = 0
    if elem.find("data") == None:
      cf = CrossCoverageFrame(elem) 
      elem.find("data").text = cf.getDataString()

    for v in [int(d) for d in elem.find("data").text.strip().split(' ')]:
      if v >= at_least:  ok += 1
      if v >= 0:       expected += 1
      if v == -1:      named += 1
      if v == -2:      excluded += 1
      if v == -3:      illegal += 1
    return expected, excluded, ok, named, illegal

  def rdint(v): return int(round(v))
  def getWeight(cp_cc):
    w = cp_cc.attrib["weight"]
    if w == None or w =="": return 0
    else: return int(w)

  for cp in maintree.xpath("//cp"):
    at_least = int(cp.attrib["at_least"])
    expected, excl, ok = scoring_bns(cp, at_least)
    cp.attrib["ok"] = str(ok)
    cp.attrib["expected"] = str(expected)
    cp.attrib["excluded"] = str(excl)
    if expected>0: cp.attrib["score"] = str(round(100.0*ok/expected, 2))
  
  for cc in maintree.xpath("//cc"):
    at_least = int(cc.attrib["at_least"])
    named_bn_result = scoring_bns(cc, at_least)
    if len(cc.xpath("namedbins")) == 0:
      namedbin = maintree.makeelement('namedbins')
      cc.append(namedbin)
    else:
      namedbin = cc.xpath("namedbins")[0]
    namedbin.attrib["expected"] = str(named_bn_result[0])
    namedbin.attrib["excluded"] = str(named_bn_result[1])
    namedbin.attrib["ok"]     = str(named_bn_result[2])
    if named_bn_result[0] > 0:
      namedbin.attrib["score"]  = str(round(100.0 * named_bn_result[2]/named_bn_result[0], 2))
  
    auto_bn_result =  scoring_auto_crosses(cc, at_least)
    data = cc.find("data")
    data.attrib["expected"] = str(auto_bn_result[0])
    data.attrib["excluded"] = str(auto_bn_result[1])
    data.attrib["ok"]     = str(auto_bn_result[2])
    data.attrib["named"]  = str(auto_bn_result[3])
    data.attrib["illegal"]  = str(auto_bn_result[4])
    if auto_bn_result[0] > 0:
      data.attrib["score"] = str(round(100.0 * auto_bn_result[2]/auto_bn_result[0],2))

    cc.attrib["expected"] = str(named_bn_result[0] + auto_bn_result[0])
    cc.attrib["excluded"] = str(named_bn_result[1] + auto_bn_result[1])
    cc.attrib["ok"]     = str(named_bn_result[2] + auto_bn_result[2])
    cc.attrib["weight"]   = cc.attrib["expected"]
    if named_bn_result[0]+auto_bn_result[0] > 0:
      cc.attrib["score"] = str(round(100.0 * (named_bn_result[2] + auto_bn_result[2]) /
                    (named_bn_result[0] + auto_bn_result[0]),2))

  ## score cumulative/inst
  for cum_inst in maintree.xpath("//cumulative|//inst"):
    weight_sum = score_sum = 0
    for cp_cc in cum_inst.xpath("cp|cc"):
      weight = getWeight(cp_cc)
      weight_sum += weight
      if "score" in cp_cc.attrib:
        score = float(cp_cc.attrib["score"])
        score_sum += score * weight

    if cum_inst.attrib.get("weight") == "":
      cum_inst.attrib["weight"] = unicode(weight_sum)
    if weight_sum > 0:
      cum_inst.attrib["score"]  = str(round(score_sum/weight_sum, 2))
        
  ## score sum up for each cumulative/inst 
  for mode in ("cumulative", "inst"):
    total_score  = 0.0
    total_weight = 0
    cum_inst_nodes = maintree.xpath("//cg/%s[@score][@weight]"%mode)
    total_weight = sum([getWeight(cum_inst) for cum_inst in cum_inst_nodes])
    for cum_inst in cum_inst_nodes:
      weight = getWeight(cum_inst) 
      score =   float(cum_inst.attrib["score"])
      total_score  += score * weight
      cum_inst.attrib["uncovered"] = str(round(((100.0-score)*weight/total_weight),3))

      weight_ratio = weight / total_weight
      cp_ccs = cum_inst.xpath("cp[@score]|cc[@score]")
      total_cc_cp_weight = sum([getWeight(cp_cc) for cp_cc in cp_ccs])
      for cc_cp in cp_ccs:
        s = float(cc_cp.attrib["score"])
        w = getWeight(cc_cp)
        cc_cp.attrib["uncovered"] = str(round((100.0 - s) * w * weight_ratio / total_cc_cp_weight, 2))
    if total_weight >0:  
      maintree.attrib["score_"+mode] = str(round(total_score/total_weight, 2))
  
  ## score sum up inst nodes 
  cg_nodes = maintree.xpath("//cg")
  for cg in cg_nodes:
    total_score  = 0.0
    total_weight = 0
    total_uncovered = 0.0
    for inst in cg.xpath("inst"):
      weight = getWeight(inst)
      score  = float(inst.attrib["score"])
      total_score  += weight * score
      total_weight += weight
      total_uncovered += float(inst.attrib["uncovered"])
    if total_weight>0:
      cg.attrib["inst_score"] = str(round(total_score / total_weight, 2))
      cg.attrib["inst_uncovered"] = str(round(total_uncovered,2))

def processAttribMod(targets, mod):
  for cp in targets:
    assert(cp.tag.endswith("cp") or cp.tag.endswith("cc"))
    for key,value in mod["attrib"].iteritems():
      if key not in ("weight", "at_least", "description"):
        errprint("WARNING VOID REWRITING:%s -> %s @%s/%s"%(key,value, mod["cg"], mod["cp"]))
      cp.attrib[key] = unicode(value)

def processCgAttribMod(targets, mod):
  for cg in targets:
    assert(cg.tag.endswith("cumulative") or cg.tag.endswith("inst"))
    for key,value in mod["attrib"].iteritems():
      if key not in ("weight", "class", "description"):
        errprint("WARNING VOID REWRITING:%s -> %s @%s"%(key,value, mod["cg"]))
      cg.attrib[key] = unicode(value)

def processBnMod(targets, mod):
  for cp in targets:
    for bn_name, bn_mod in mod["bn"].iteritems():
      bns = cp.xpath("bn" if bn_name=="*" else 'bn[@name="%s"]'%bn_name)
      if len(bns)==0:
        errprint("WARNING NO SUCH BINS %s/%s/%s"%(mod["cg"], mod["cp"],bn_name))
        continue
      for bn in bns:
        for key,value in bn_mod.iteritems():
          if(key not in ("excl")):
            errprint("WARNING INVALID REWRITING %s"%key)
            return
          bn.attrib[key]=value
    if "weight" not in cp.attrib or cp.attrib["weight"] == "":
      cp.attrib["weight"] = unicode(len(cp.xpath("bn")) - len(cp.xpath('bn[@excl="true"]')))
    
def processCrossCoverageMod(targets, mod):
  for cc in targets:
    if not cc.tag.endswith("cc"): continue
    ##print("MODIFYING %s, %s"%(mod["cg"], mod["cc"]))
    cf = cross_coverage_frame_indice.get((mod["cg"], mod["cc"]))
    if cf==None:
      cf = CrossCoverageFrame(cc)
      path = "%s/%s"%(mod["cg"], mod["cc"])
      pathToNode[path] = cc
      cross_coverage_frames[path] = cf

    bn = maintree.makeelement('bn')
    bn.attrib["name"] = mod["name"]
    bn.attrib["excl"] = "true" if mod["type"] in ("excluded", "illegal") else "false"
    for comb in mod["combinations"]:
      dom = maintree.makeelement('domain')
      for span in comb:
        if isinstance(span, str) or isinstance(span, unicode):
          description = span
        elif isinstance(span, list):
          description = " ".join(span)
        elif isinstance(span, dict) and "from" in span and "to" in span:
          description = "%s to %s"%(span["from"], span["to"])
        else:
          description = ""
          errprint("Invalid combination" + str(comb))
          
        axis = maintree.makeelement('axis')
        axis.attrib["description"] = description
        dom.append(axis)
      bn.append(dom)
    cc.append(bn)

    count = cf.modifyAndSumBinDatas(mod["axes"], mod["combinations"], {"named":-1, "excluded":-2, "illegal":-3}[mod["type"]])
    bn.attrib["count"] = str(count)

    data = cc.find('data')
    assert(data!=None)
    data.text = " ".join(map(str, cf.dat))


def modifyXmlTree(mod_file):
  def all_in(s1, s2): return reduce(lambda b,s: b and s in s2, s1, True)
  import json
  if isinstance(mod_file,str): mod_file = open(mod_file)
  mods = json.load(mod_file)
  for mod in mods:
    ##CHECK JSON FILE
    if "cg" not in mod:
      errprint("WARNING: modification ignored: INCOMPLETE MODIFICATION\n\tNo Cover Group" + str(mod)) 
      continue
    if "cp" not in mod and "cc" not in mod:
      if "attrib" in mod:
        targets = maintree.xpath('//cumulative[@name="%s"]|//inst[@name="%s"]'%(mod["cg"], mod["cg"]))
        if len(targets) > 0:
          processCgAttribMod(targets, mod)
          continue
      errprint("WARNING: modification ignored: INCOMPLETE MODIFICATION\n\tNo Coverage Point" + str(mod)) 
      continue
    if "attrib" not in mod and "bn" not in mod and not all_in(("cc", "axes", "combinations", "type", "name"), mod):
      errprint("WARNING: modification ignored: INCOMPLETE CROSS COVERAGE MOD\n\t" + str(mod))
      continue

    cp_name = mod.get("cp") or mod["cc"]
    targets = maintree.xpath('//*[@name="%s"]/*[@name="%s"]'%(mod["cg"],cp_name))
    if len(targets) == 0:
      errprint("WARNING: modification ignored. No such target %s/%s"%(mod["cg"], cp_name))
      continue

    ##PROCESS MODIFICATION
    if "attrib" in mod: processAttribMod(targets, mod)
    if "bn" in mod: processBnMod(targets, mod)
    if "combinations" in mod: processCrossCoverageMod(targets, mod)


def finalize():
  """write count bn datas."""
  for path,cc_frame in cross_coverage_frames.items():
    pathToNode[path].find("data").text = cc_frame.getDataString()
  doScoring()
  dump(output)

def start(stream, mod_file=None):
  global maintree
  if stream.name.endswith(".xml"):
    maintree = etree.parse(stream).getroot()
  else:
    maintree = etree.Element("testbench")
    scanStream(stream)
  if mod_file != None:
    modifyXmlTree(mod_file)
  finalize()

def dump(stream = stdout):
  stream.write(etree.tostring(maintree, pretty_print=True))

def startTest():
  global output
  print("START SELF TESTING")
  output = open("result.test1-1.xml",'w')
  start(open("sample/jsontest1/result.xml"), "sample/jsontest1/carp2_odfpnl_mod.json")
  output = open("result.test1-2.xml",'w')
  start(open("sample/jsontest1/result.xml"), "sample/jsontest1/mod2.json")
  output = open("result.test1-3.xml",'w')
  start(open("sample/jsontest1/result.xml"), "sample/jsontest1/mod3.json")
  print("START END TESTING")


if __name__ =='__main__':
  import argparse
  parser = argparse.ArgumentParser(description='Process VCS/UCDB result file')
  #parser.add_argument('--vcs', action='store_true', default=False, help='process result of vericlear_vcs')
  parser.add_argument('inputfile', nargs='?', type=argparse.FileType('r'), default=stdin, help='input file(default=stdin)')
  parser.add_argument('-o','--outputfile', type=argparse.FileType('w'), default=stdout, help='output xml file(default=result.xml)')
  parser.add_argument('modfile', nargs='?', type=argparse.FileType('r'), help='mod file')
  parser.add_argument('--test', action='store_true', default=False)
  parser.add_argument('-v', '--verbose', action='store_true', default=False)
  ns = parser.parse_args(argv[1:])

  output = ns.outputfile
  #vcs_mode = ns.vcs
  verbose = ns.verbose

  if ns.test:
    startTest()
  elif ns.modfile != None:
    start(ns.inputfile)
  else:
    start(ns.inputfile, ns.modfile)
