//----------------------------------------------------------------------
/**
* @file vericler_ucdb.cpp
* @brief ucdb preprocessor for vericlear
*/
/*
* Copyright (C) 2007-2012 Verifore, Inc.
* All rights reserved. Property of Verifore, Inc.
* Restricted rights to use, duplicate or disclose
* this code are granted through contract.
*/
//----------------------------------------------------------------------
        
        
#include "ucdb.h"
#include <iostream>
#include <map>
#include <string>
#include <assert.h>
#include <string.h>

using namespace std;

map<uint64_t, string> space_map(){
	map<uint64_t, string> retmap;
	retmap.insert(pair<uint64_t, string>(UCDB_TOGGLE, "TOGGLE"));
	retmap.insert(pair<uint64_t, string>(UCDB_BRANCH, "BRANCH"));
	retmap.insert(pair<uint64_t, string>(UCDB_EXPR, "EXPR"));
	retmap.insert(pair<uint64_t, string>(UCDB_COND, "COND"));
	retmap.insert(pair<uint64_t, string>(UCDB_INSTANCE, "INSTANCE"));
	retmap.insert(pair<uint64_t, string>(UCDB_PROCESS, "PROCESS"));
	retmap.insert(pair<uint64_t, string>(UCDB_BLOCK, "BLOCK"));
	retmap.insert(pair<uint64_t, string>(UCDB_FUNCTION, "FUNCTION"));
	retmap.insert(pair<uint64_t, string>(UCDB_FORKJOIN, "FORKJOIN"));
	retmap.insert(pair<uint64_t, string>(UCDB_GENERATE, "GENERATE"));
	retmap.insert(pair<uint64_t, string>(UCDB_GENERIC, "GENERIC"));
	retmap.insert(pair<uint64_t, string>(UCDB_CLASS, "CLASS"));
	retmap.insert(pair<uint64_t, string>(UCDB_COVERGROUP, "COVERGROUP"));
	retmap.insert(pair<uint64_t, string>(UCDB_COVERINSTANCE, "COVERINSTANCE"));
	retmap.insert(pair<uint64_t, string>(UCDB_COVERPOINT, "COVERPOINT"));
	retmap.insert(pair<uint64_t, string>(UCDB_CROSS, "CROSS"));
	retmap.insert(pair<uint64_t, string>(UCDB_COVER, "COVER"));
	retmap.insert(pair<uint64_t, string>(UCDB_ASSERT, "ASSERT"));
	retmap.insert(pair<uint64_t, string>(UCDB_PROGRAM, "PROGRAM"));
	retmap.insert(pair<uint64_t, string>(UCDB_PACKAGE, "PACKAGE"));
	retmap.insert(pair<uint64_t, string>(UCDB_TASK, "TASK"));
	retmap.insert(pair<uint64_t, string>(UCDB_INTERFACE, "INTERFACE"));
	retmap.insert(pair<uint64_t, string>(UCDB_FSM, "FSM"));
	retmap.insert(pair<uint64_t, string>(UCDB_TESTPLAN, "TESTPLAN"));
	retmap.insert(pair<uint64_t, string>(UCDB_DU_MODULE, "DU_MODULE"));
	retmap.insert(pair<uint64_t, string>(UCDB_DU_ARCH, "DU_ARCH"));
	retmap.insert(pair<uint64_t, string>(UCDB_DU_PACKAGE, "DU_PACKAGE"));
	retmap.insert(pair<uint64_t, string>(UCDB_DU_PROGRAM, "DU_PROGRAM"));
	retmap.insert(pair<uint64_t, string>(UCDB_DU_INTERFACE, "DU_INTERFACE"));
	retmap.insert(pair<uint64_t, string>(UCDB_FSM_STATES, "FSM_STATES"));
	retmap.insert(pair<uint64_t, string>(UCDB_FSM_TRANS, "FSM_TRANS"));
	retmap.insert(pair<uint64_t, string>(UCDB_RESERVEDSCOPE, "RESERVEDSCOPE"));
	retmap.insert(pair<uint64_t, string>(UCDB_SCOPE_ERROR, "SCOPE_ERROR"));
	retmap.insert(pair<uint64_t, string>(UCDB_FSM_SCOPE, "FSM_SCOPE"));
	retmap.insert(pair<uint64_t, string>(UCDB_CODE_COV_SCOPE, "CODE_COV_SCOPE"));
	retmap.insert(pair<uint64_t, string>(UCDB_DU_ANY, "DU_ANY"));
	retmap.insert(pair<uint64_t, string>(UCDB_CVG_SCOPE, "CVG_SCOPE"));
	retmap.insert(pair<uint64_t, string>(UCDB_FUNC_COV_SCOPE, "FUNC_COV_SCOPE"));
	retmap.insert(pair<uint64_t, string>(UCDB_COV_SCOPE, "COV_SCOPE"));
	retmap.insert(pair<uint64_t, string>(UCDB_VERIF_SCOPE, "VERIF_SCOPE"));
	retmap.insert(pair<uint64_t, string>(UCDB_HDL_SUBSCOPE, "HDL_SUBSCOPE"));
	retmap.insert(pair<uint64_t, string>(UCDB_HDL_INST_SCOPE, "HDL_INST_SCOPE"));
	retmap.insert(pair<uint64_t, string>(UCDB_HDL_DU_SCOPE, "HDL_DU_SCOPE"));
	retmap.insert(pair<uint64_t, string>(UCDB_HDL_SCOPE, "HDL_SCOPE"));
	retmap.insert(pair<uint64_t, string>(UCDB_NONTESTPLAN_SCOPE, "NONTESTPLAN_SCOPE"));
	retmap.insert(pair<uint64_t, string>(UCDB_NO_SCOPES, "NO_SCOPES"));
	retmap.insert(pair<uint64_t, string>(UCDB_ALL_SCOPES, "ALL_SCOPES"));

	return retmap;
}
map<uint64_t, string> gScopeTypeMap = space_map();

void print_tags(ucdbT db, ucdbScopeT scope)
{
	int i, ntags;
	ntags = ucdb_GetObjNumTags(db, (ucdbObjT)scope);
	if(ntags==0) return;
	const char* tagname;
	for(int i=0; i<ntags; i++){
		ucdb_GetObjIthTag(db,(ucdbObjT)scope, i, &tagname);
		std::cout << "[" << tagname << "]";
	}
	std::cout << std::endl;
}

void print_data(ucdbT db, ucdbScopeT scope, int coverindex)
{
	const char* attrname;
	ucdbAttrValueT* attrvalue;
	ucdbCoverDataT covdata;
	char* covername;
	int attrs = 0;
	int scope_weight;

	ucdbScopeTypeT stype;
	if(coverindex>=0){ // BIN
		ucdb_GetCoverData(db, scope, coverindex, &covername, &covdata, NULL);
		std::cout << "BN " << "path=" << ucdb_GetScopeHierName(db, scope) << ",covname=" << covername;

		cout << ",typeflag=" << (void*)covdata.type;
		if     (covdata.type & UCDB_ILLEGALBIN) cout << ",illegal=";
		if     (covdata.type & UCDB_IGNOREBIN) cout << ",ignored=";

		if     (covdata.flags & UCDB_IS_32BIT) cout << ",val=" << covdata.data.int32;
		else if(covdata.flags & UCDB_IS_64BIT) cout << ",val=" << covdata.data.int64;
		else if(covdata.flags & UCDB_IS_VECTOR) assert(false); //"vector data not supported"

		if(covdata.flags & UCDB_HAS_GOAL) cout << ",goal=" << covdata.goal;
		if(covdata.flags & UCDB_HAS_WEIGHT) cout << ",weight=" << covdata.weight;
		if(covdata.flags & UCDB_HAS_LIMIT) cout << ",limit=" << covdata.limit;
		if(covdata.flags & UCDB_EXCLUDED) cout << ",excluded=";
		if(covdata.flags & UCDB_ENABLED==0) cout << ",disabled=";
		

	} else if(coverindex==-1){
		std::cout << "SC " << "path=" << ucdb_GetScopeHierName(db, scope);
		stype = ucdb_GetScopeType(db,scope);
		map<uint64_t, string>::iterator p = gScopeTypeMap.find(stype);
		if(p != gScopeTypeMap.end())
			cout << ",scopetype=" << p->second;
		if((scope_weight = ucdb_GetScopeWeight(db, scope)) >=0)
			cout << ",weight=" << ucdb_GetScopeWeight(db, scope);
		//int flags = ucdb_GetScopeFlags(db, scope);
		//if(flags | UCDB_SCOPE_EXCLUDED!=0) cout << ",excluded=";
	}
	cout << ",";
	attrname = NULL;

	while((attrname = ucdb_AttrNext(db, (ucdbObjT)scope, coverindex, attrname, &attrvalue))) {
		attrs++;
		std::cout << attrname << "=";
		switch(attrvalue->type){
			case UCDB_ATTR_INT: std::cout << attrvalue->u.ivalue; break;
			case UCDB_ATTR_FLOAT: std::cout << attrvalue->u.fvalue; break;
			case UCDB_ATTR_DOUBLE: std::cout << attrvalue->u.dvalue; break;
			case UCDB_ATTR_INT64: std::cout << attrvalue->u.i64value; break;
			case UCDB_ATTR_STRING: std::cout << attrvalue->u.svalue; break;
			case UCDB_ATTR_MEMBLK:
				std::cout << "bin size=" << attrvalue->u.mvalue.size;
				for(int i=0; i<attrvalue->u.mvalue.size; i++){
					std::cout << attrvalue->u.mvalue.data[i];
				}
				break;
			default:
				std::cout << "ERROR ATTRVAL" << attrvalue->type;
		}
		std::cout << ",";
	}
	if(coverindex==-1 && stype == UCDB_CROSS){
		attrs++;
		int n_crosses;
		ucdb_GetNumCrossedCvps(db, scope, &n_crosses);
		cout << "CROSSES=[";
		for(int i=0; i<n_crosses; i++){
			std::cout << ucdb_GetIthCrossedCvpName(db,scope,i) << ",";
		}
		cout << "]";
	}
	std::cout << std::endl;
}

void print_test(ucdbT db, ucdbScopeT scope)
{

}

ucdbCBReturnT callback(void* userdata, ucdbCBDataT* cbdata)
{
	ucdbScopeT scope = (ucdbScopeT)(cbdata->obj);
	ucdbT db = cbdata->db;
	char* name;
	ucdbCoverDataT coverdata;
	ucdbSourceInfoT sourceinfo;
	char* str;
	
	switch(cbdata->reason){
	case UCDB_REASON_DU:
		scope = (ucdbScopeT)(cbdata->obj);
		str = ucdb_GetInstanceDUName(db, scope);
		return UCDB_SCAN_PRUNE;
	case UCDB_REASON_CVBIN:
		scope = (ucdbScopeT)(cbdata->obj);
		print_data(cbdata->db, scope, cbdata->coverindex);
		break;
	case UCDB_REASON_SCOPE:
		scope = (ucdbScopeT)(cbdata->obj);
		print_data(cbdata->db, scope, -1);
		break;
	case UCDB_REASON_TEST:
		scope = (ucdbScopeT)(cbdata->obj);
		//std::cout << "TEST" << std::endl;
		break;
	default:
		//std::cout << "other reason" << cbdata->reason << std::endl;
		break;
	}
	return UCDB_SCAN_CONTINUE;
}

int main(int argc, const char** argv)
{
	bool b_bin_only = false;
	const char* filename = NULL;
	for(int i=1; i<argc; i++){
		if(strcmp(argv[i], "-b")==0)
      b_bin_only = true;
		else {
      filename = argv[i];

      if(filename==NULL){
        cerr << "File name not found." << endl; return -1;
      }

      ucdbT db = ucdb_Open(filename);
      if(db == NULL){
        std::cerr << "ERROR OPENING FILE" << std::endl;
        return -1;
      }
      std::cout << "#VERICLEAR UCDB" << std::endl;
      if(b_bin_only)
        ucdb_PathCallBack(db, 1, NULL, NULL, UCDB_NONTESTPLAN_SCOPE,
          0, UCDB_CVGBIN, callback, NULL);
      else
        ucdb_CallBack(db, NULL, callback, NULL);
      b_bin_only = true; // force bin only mode after first load
      ucdb_Close(db);
    }
  }
	return 0;
}
