//----------------------------------------------------------------------
/**
* @file vericler_ucdb.cpp
* @brief Synopsys VCS/Ucapi preprocessor for vericlear
*/
/*
* Copyright (C) 2007-2012 Verifore, Inc.
* All rights reserved. Property of Verifore, Inc.
* Restricted rights to use, duplicate or disclose
* this code are granted through contract.
*/
//----------------------------------------------------------------------

#include <covdb_user.h>
#include <iostream>
#include <fstream>
#include <string>

using namespace std;
ofstream log("vericlear_ucapi_log.txt");

void dump(covdbHandle handle, covdb1ToManyRelationsT relation) {
  covdbHandle itr = covdb_iterate(handle, relation);
  covdbHandle h;
  while((h = covdb_scan(itr))){
    log << "dump: " << covdb_get_str(h, covdbName) << endl;
  }
  covdb_release_handle(itr);
}

void error_cb(covdbHandle err, void *data)
{
  int err_code = covdb_get(err, NULL, NULL, covdbValue);
  char* msg = covdb_get_str(err, covdbName);

  cerr << "ERROR OCCURED:" << msg << endl;
}

string getComment(covdbHandle h)
{
  char* s = covdb_get_annotation(h, COV_COMMENT);
  if(s!=NULL) return string(s);
  else return string("");
}

void aboutCP(covdbHandle cp, covdbHandle cg, covdbHandle metric, covdbHandle test, string path)
{
  int is_cross = covdb_get_integer_annotation(cp, "isCross");
  log << "      3:" << covdb_get_str(cp, covdbName);
  log << " iscross=" << covdb_get_integer_annotation(cp, "isCross");
  log << " type=" << covdb_get(cp, cg, test, covdbType) << "," << covdbValueSet;
  log << " weight=" << covdb_get(cp, cg, test, covdbWeight);
  log << " atleast=" << covdb_get(cp, cg, test, covdbCovCountGoal);
  log << " comment=" << covdb_get_qualified_annotation(cp, metric, COV_COMMENT);
  log << endl;

  cout << "SC path=" << path;
  cout << ",scopetype=" << ((is_cross)?"CROSS": "COVERPOINT");
  cout << ",weight=";
  cout << covdb_get(cp, cg, test, covdbWeight);
  cout << ",ATLEAST=" << covdb_get(cp, cg, test, covdbCovCountGoal);
  if(is_cross){
    cout << ",CROSSES=[";
    covdbHandle h,hs = covdb_iterate(cp, covdbComponents);
    while((h = covdb_scan(hs))) {
      cout << covdb_get_str(h, covdbName) << ",";
    }
    cout << "]";
  }
  cout << ",COMMENT=" << covdb_get_qualified_annotation(cp, metric, COV_COMMENT);
  cout << endl;
}
void scanCP(covdbHandle cp, covdbHandle cg, covdbHandle test, string path)
{
  covdbHandle cp2, cps2 = covdb_iterate(cp, covdbObjects);
  while((cp2 = covdb_scan(cps2))) {
    log << "        4:" << covdb_get_str(cp2, covdbName);
    log << endl;

    covdbHandle bn, bns = covdb_iterate(cp2, covdbObjects);
    while((bn = covdb_scan(bns))) {
      log << "          5:" << covdb_get_str(bn, covdbName);
      log << " count=" << covdb_get(bn, cg, test, covdbCovCount);
      log << " status=" << covdb_get(bn, cg, test, covdbCovStatus);
      log << " type=" << covdb_get(bn, cg, test, covdbType) << "," << covdbValueSet;
      ////log << ">" << covdb_get(bn, cg, test, covdbWeight) << endl;
      log << " cp_bin=" << covdb_get_integer_annotation(bn, "cp_bin");
      log << endl;
      
      int status = covdb_get(bn, cg, test, covdbCovStatus);
      cout << "BN path=" << path;
      cout << ",covname=" << covdb_get_str(bn,covdbName);
      cout << ",val=" << covdb_get(bn, cg, test, covdbCovCount);
      if(status & covdbStatusExcluded) cout << ",excluded=";
      if(status & covdbStatusIllegal)  cout << ",illegal=";
      cout << endl;
      covdbHandle bn2, bns2 = covdb_iterate(bn, covdbObjects);
    }
    covdb_release_handle(bns);
  }
}

void scanCC(covdbHandle cc, covdbHandle cg, covdbHandle test, string path)
{
  covdbHandle cp2, cps2 = covdb_iterate(cc, covdbObjects);
  while((cp2 = covdb_scan(cps2))) {
    log << "        4:" << covdb_get_str(cp2, covdbName);
    log << endl;
    int automatic_id = 0;
    char buff[8];

    covdbHandle bn, bns = covdb_iterate(cp2, covdbObjects);
    while((bn = covdb_scan(bns))) {
      int status = covdb_get(bn, cg, test, covdbCovStatus);
      int excluded = (status & covdbStatusExcluded)?1:0;
      int illegal = (status & covdbStatusIllegal)? 1:0;

      log << "          5:" << covdb_get_str(bn, covdbName);
      log << " count=" << covdb_get(bn, cg, test, covdbCovCount);
      log << " excluded=" << excluded;
      log << " illegal=" << illegal;
      ////log << ">" << covdb_get(bn, cg, test, covdbWeight) << endl;
      //log << " type=" << covdb_get_integer_annotation(bn, "cp_bin");
      log << " automatic=" << covdb_get(bn, cg, test, covdbAutomatic);
      log << " status=" << status;
      log << endl;

      cout << "BN path=" << path;
      int automatic = covdb_get(cp2, cg, test, covdbAutomatic);

      covdbHandle bn2, bns2;
      if(automatic) {
        bns2 = covdb_iterate(bn, covdbComponents);
        int bns_count;
        cout << ",covname=[";
        while((bn2 = covdb_scan(bns2))) {
          log << "              6a:" << covdb_get_str(bn2, covdbName);
          log << " automatic=" << covdb_get(bn, cg, test, covdbAutomatic);
          //log << " count=" <<  covdb_get(bn2, cg, test, covdbCovCount);
          log << " status=" << covdb_get(bn2, cg, test, covdbCovStatus);
          log << endl;
          cout << covdb_get_str(bn2, covdbName) << ",";
          
          covdbHandle h,hs = covdb_iterate(bn2, covdbObjects);
          while((h = covdb_scan(hs))) {
            log << "                7a:" << covdb_get_str(h, covdbName) << endl;
            //log << covdb_get_str(h, covdbValueName) << endl;
          }
          covdb_release_handle(hs);
        }
        cout << "]";
        covdb_release_handle(bns2);
      } else {
        bns2 = covdb_iterate(bn, covdbObjects);
        cout << ",covname=" << covdb_get_str(bn,covdbName);

        cout << ",BINRHS=[";
        while((bn2 = covdb_scan(bns2))) {
          log << "              6b:" << covdb_get_str(bn2, covdbName);
          //log << " count=" <<  covdb_get(bn2, cg, test, covdbCovCount);
          //log << " status=" << covdb_get(bn2, cg, test, covdbCovStatus);
          log << endl;
          
          covdbHandle h,hs = covdb_iterate(bn2, covdbObjects);
          while((h = covdb_scan(hs))) {
            log << "                7b:" << covdb_get_str(h, covdbName) << endl;
            //log << covdb_get_str(h, covdbValueName) << endl;
            cout << covdb_get_str(h,covdbName) << ";";
          }
          covdb_release_handle(hs);
        }
        cout << "]";
        covdb_release_handle(bns2);
      }
      cout << ",val=" << covdb_get(bn, cg, test, covdbCovCount);
      if(excluded) cout << ",excluded=";
      if(illegal)  cout << ",illegal=";
      cout << endl;
    }
    covdb_release_handle(bns);
  }
  covdb_release_handle(cps2);
}


void scanGroup(covdbHandle cg, covdbHandle metric, covdbHandle test)
{
  covdbHandle cg2,cgs2 = covdb_qualified_iterate(cg, metric, covdbDefinitions);
  while((cg2 = covdb_scan(cgs2))) {
    log << "    2:" << covdb_get_str(cg2, covdbName);
    log << " weight=" << covdb_get(cg2, cg, test, covdbWeight);
    log << " comment=" << covdb_get_qualified_annotation(cg2, metric, COV_COMMENT);
    log << endl;

    string path = string(covdb_get_str(cg2, covdbName));
    cout << "SC path=" << path << ",scopetype=COVERGROUP,weight=";
    cout << covdb_get(cg2, cg, test, covdbWeight);
    cout << ",comment=" << covdb_get_qualified_annotation(cg2, metric, COV_COMMENT);
    cout << endl;

    covdbHandle cp,cps = covdb_iterate(cg2, covdbObjects);
    while((cp = covdb_scan(cps))) {
      string child_path = path + "/" + covdb_get_str(cp, covdbName);
      aboutCP(cp, cg2, metric, test, child_path);
      int is_cross = covdb_get_integer_annotation(cp, "isCross");
      if(is_cross) {
        scanCC(cp, cg2, test, child_path);
      } else {
        scanCP(cp, cg2, test, child_path);
      }
    }
    covdb_release_handle(cps);
  }
  covdb_release_handle(cgs2);
}

void scanInst(covdbHandle inst, covdbHandle metric, covdbHandle test)
{
  covdbHandle h = covdb_get_handle(inst, covdbDefinition);
  log << "  i1:" << covdb_get_str(inst, covdbName);
  log << " fullname=" << covdb_get_str(inst, covdbFullName);
  log << " definitions=" << covdb_get_str(h, covdbName);
  log << " weight="  << covdb_get(inst, metric, test, covdbWeight);
  log << " comment=" << covdb_get_qualified_annotation(inst, metric, COV_COMMENT);
  log << endl;

  string path = string(covdb_get_str(h,covdbName)) + string("/") + string(covdb_get_str(inst, covdbName));
  cout << "SC path=" <<  path;
  cout << ",scopetype=COVERINSTANCE,weight=" << covdb_get(inst, metric, test, covdbWeight);
  cout << ",COMMENT=" << covdb_get_qualified_annotation(inst, metric, COV_COMMENT);
  cout << endl;

  covdbHandle cp,cps = covdb_iterate(inst, covdbObjects);
  while((cp = covdb_scan(cps))) {
    log << "      i2:" << covdb_get_str(cp, covdbName);
    log << " iscross=" << covdb_get_integer_annotation(cp, "isCross");
    log << " type=" << covdb_get(cp, inst, test, covdbType) << "," << covdbValueSet;
    log << " weight=" << covdb_get(cp, inst, test, covdbWeight);
    log << " atleast=" << covdb_get(cp, inst, test, covdbCovCountGoal);

    string child_path = path + "/" + covdb_get_str(cp, covdbName);
    cout << "SC path=" << child_path;
    cout << ",scopetype=" << ((covdb_get_integer_annotation(cp, "isCross"))?"CROSS": "COVERPOINT");
    cout << ",weight=" << covdb_get(cp, inst, test, covdbWeight);
    cout << ",ATLEAST=" << covdb_get(cp, inst, test, covdbCovCountGoal);
    cout << ",COMMENT=" << covdb_get_qualified_annotation(cp, metric, COV_COMMENT);
    cout << endl;

    if(covdb_get_integer_annotation(cp, "isCross")) {
      scanCC(cp, inst, test, child_path);
    } else {
      scanCP(cp, inst, test, child_path);
    }
  }
  covdb_release_handle(cps);
}


int main(int argc, char* argv[]){
  covdbHandle test, h_design;
  covdbHandle availableTestName;
  char *error_msg;

  covdb_set_error_callback(error_cb, NULL);

  if(argc == 1){
    h_design = covdb_load(covdbDesign, NULL, "simv.vdb");
  } else {
    h_design = covdb_load(covdbDesign, NULL, argv[1]);
    for(int i=2; i<argc-1; i++){
      log << "Merge loading: " << argv[i] << endl;
      h_design = covdb_loadmerge(covdbDesign, h_design, argv[i]);
    }
  }
    
  covdbHandle availableTestNameIterator = covdb_iterate(h_design, covdbAvailableTests);
  cout << "#VERICLEAR VCS" << endl;

  bool b_first = true;

  while((availableTestName = covdb_scan(availableTestNameIterator))!=NULL){
    if(b_first) {
      test = covdb_load(covdbTest, h_design, covdb_get_str(availableTestName, covdbName));
      b_first = false;
    } else { 
      test = covdb_loadmerge(covdbTest, test, covdb_get_str(availableTestName, covdbName));
    }
    log << "test_name:" << covdb_get_str(test, covdbName) << endl;

    covdbHandle metric,metrics = covdb_iterate(test, covdbMetrics);
    while((metric = covdb_scan(metrics))) {
      log << "metrics:" << covdb_get_str(metric, covdbName) << endl;
      covdbHandle cg,cgs = covdb_qualified_iterate(test, metric, covdbDefinitions);
      //covdbHandle instances = covdb_iterate(metric, covdbInstances);
      while((cg = covdb_scan(cgs))) {
        scanGroup(cg, metric, test);
      }
      covdb_release_handle(cgs);

      covdbHandle inst, insts = covdb_qualified_iterate(test, metric, covdbInstances);
      while((inst = covdb_scan(insts))) {
        scanInst(inst, metric, test);
      }
      covdb_release_handle(insts);
    }
    covdb_release_handle(metrics);
  }
  covdb_release_handle(availableTestNameIterator);

  covdb_unload(h_design);
}

